@@ -75,6 +75,7 @@ enum {
 	SIGNAL_THUMBNAIL_CHANGED,
 	SIGNAL_SAVE_PROGRESS,
 	SIGNAL_NEXT_FRAME,
+	SIGNAL_FILE_CHANGED,
 	SIGNAL_LAST
 };
 
@@ -260,12 +261,20 @@ eog_image_class_init (EogImageClass *klass)
 		g_signal_new ("next-frame",
 			      EOG_TYPE_IMAGE,
 			      G_SIGNAL_RUN_LAST,
-			      G_STRUCT_OFFSET (EogImageClass, save_progress),
+			      G_STRUCT_OFFSET (EogImageClass, next_frame),
 			      NULL, NULL,
 			      g_cclosure_marshal_VOID__INT,
 			      G_TYPE_NONE, 1,
 			      G_TYPE_INT);
 
+	signals[SIGNAL_FILE_CHANGED] = g_signal_new ("file-changed",
+						     EOG_TYPE_IMAGE,
+						     G_SIGNAL_RUN_LAST,
+						     G_STRUCT_OFFSET (EogImageClass, file_changed),
+						     NULL, NULL,
+						     g_cclosure_marshal_VOID__VOID,
+						     G_TYPE_NONE, 0);
+
 	g_type_class_add_private (object_class, sizeof (EogImagePrivate));
 }
 
@@ -283,6 +292,7 @@ eog_image_init (EogImage *img)
 	img->priv->width = -1;
 	img->priv->height = -1;
 	img->priv->modified = FALSE;
+	img->priv->file_is_changed = FALSE;
 	img->priv->status_mutex = g_mutex_new ();
 	img->priv->status = EOG_IMAGE_STATUS_UNKNOWN;
         img->priv->metadata_status = EOG_IMAGE_METADATA_NOT_READ;
@@ -1142,6 +1152,8 @@ eog_image_real_load (EogImage *img,
 				priv->file_type = gdk_pixbuf_format_get_name (format);
 			}
 
+			priv->file_is_changed = FALSE;
+
 			/* If it's non-threadsafe loader, then trigger window
  			 * showing in the end of the process. */
 			if (!priv->threadsafe_format)
@@ -1244,9 +1256,11 @@ eog_image_load (EogImage *img, EogImageData data2read, EogJob *job, GError **err
 
 #ifdef HAVE_EXIF
 	/* Check that the metadata was loaded at least once before
-	 * trying to autorotate. */
+	 * trying to autorotate. Also only an image load job should try to
+	 * autorotate an image. */
 	if (priv->autorotate && 
-	    priv->metadata_status == EOG_IMAGE_METADATA_READY) {
+	    priv->metadata_status == EOG_IMAGE_METADATA_READY &&
+	    data2read & EOG_IMAGE_DATA_IMAGE) {
 		eog_image_real_autorotate (img);
 	}
 #endif
@@ -2212,3 +2226,27 @@ eog_image_get_transform (EogImage *img)
 }
 
 #endif
+
+/**
+ * eog_image_file_changed:
+ * @img: a #EogImage
+ *
+ * Marks the image file contents as changed. Also, emits
+ * EogImage::file-changed signal.
+ **/
+void
+eog_image_file_changed (EogImage *img)
+{
+	g_return_if_fail (EOG_IS_IMAGE (img));
+
+	img->priv->file_is_changed = TRUE;
+	g_signal_emit (img, signals[SIGNAL_FILE_CHANGED], 0);
+}
+
+gboolean
+eog_image_is_file_changed (EogImage *img)
+{
+	g_return_val_if_fail (EOG_IS_IMAGE (img), TRUE);
+
+	return img->priv->file_is_changed;
+}
